package tiw5.modele;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlType;

@Entity
@XmlRootElement(name = "film")
@XmlType(propOrder = { "id", "titre", "annee", "distribution",
		"realisateursRefs", "production" })
@NamedQueries({ @NamedQuery(name = "getFilmTitreAnnee", query = "SELECT f FROM Film f WHERE f.titre=:titre AND f.annee=:annee") })
public class Film {

	// Pour JPA/JAXB
	@SuppressWarnings("unused")
	private Film() {
	}

	public Film(String titre, int annee) {
		super();
		this.titre = titre;
		this.annee = annee;
	}

	// Le couple titre/annee permet normalement d'identifier un film
	// on simplifie les choses au niveau JPA en ajoutant un identifiant
	// numerique
	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	@XmlAttribute(name = "id")
	private long id;

	private String titre;

	private int annee;

	@OneToMany(mappedBy = "film", cascade = { CascadeType.ALL })
	private List<Role> distribution = new ArrayList<Role>();

	@ManyToMany
	private List<Personne> realisateurs = new ArrayList<Personne>();

	@ManyToMany
	@XmlElement(name = "production")
	private List<Societe> production = new ArrayList<Societe>();

	@XmlElement(name = "titre", required = true)
	public String getTitre() {
		return titre;
	}

	public void setTitre(String titre) {
		this.titre = titre;
	}

	@XmlElement(name = "annee", required = true)
	public int getAnnee() {
		return annee;
	}

	public void setAnnee(int annee) {
		this.annee = annee;
	}

	public void retireRole(Role r) {
		distribution.remove(r);
	}

	public void ajouteRealisateur(Personne p) {
		realisateurs.add(p);
	}

	public void retireRealisateur(Personne p) {
		realisateurs.remove(p);
	}

	public void ajouteSocieteProduction(Societe s) {
		production.add(s);
	}

	public void removeSocieteProduction(Societe s) {
		production.remove(s);
	}

	@XmlTransient
	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

	@XmlElement(name = "role")
	public List<Role> getDistribution() {
		return distribution;
	}

	@SuppressWarnings("unused")
	private void setDistribution(List<Role> distrib) {
		this.distribution = distrib;
	}

	@XmlTransient
	public List<Personne> getRealisateurs() {
		return realisateurs;
	}

	@XmlTransient
	public List<Societe> getProduction() {
		return production;
	}

	void rawAddRole(Role r) {
		distribution.add(r);
	}

	void rawRemoveRole(Role r) {
		distribution.remove(r);
	}

	public Role ajouteRole(Personne p) {
		return new Role(p, this);
	}

	private static class PersonneRef {
		@XmlAttribute(name = "personneId")
		public String personneId;

		public Personne toPersonne() {
			return new Personne(personneId, null);
		}

		public static PersonneRef fromPersonne(Personne p) {
			PersonneRef pr = new PersonneRef();
			pr.personneId = p.getId();
			return pr;
		}
	}

	@SuppressWarnings("unused")
	@XmlElement(name = "realisateur")
	private List<PersonneRef> getRealisateursRefs() {
		List<PersonneRef> lpr = new ArrayList<Film.PersonneRef>();
		for (Personne p : getRealisateurs()) {
			lpr.add(PersonneRef.fromPersonne(p));
		}
		return lpr;
	}

	@SuppressWarnings("unused")
	private void setRealisateursRefs(List<PersonneRef> lpr) {
		realisateurs = new ArrayList<Personne>();
		for (PersonneRef pr : lpr) {
			realisateurs.add(pr.toPersonne());
		}
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (int) (id ^ (id >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Film other = (Film) obj;
		if (id != other.id)
			return false;
		return true;
	}
}
