package tiw5.modele;

import java.util.Date;
import java.util.GregorianCalendar;

import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlSchemaType;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlType;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

@Entity
@XmlRootElement(name="personne")
@XmlType(propOrder={"id","nom","DN"})
public class Personne {

	// Pour JPA/JAXB
	@SuppressWarnings("unused")
	private Personne () {}
	
	public Personne(String id, String nom) {
		super();
		this.id = id;
		this.nom = nom;
	}

	/** Identifiant d'une personne, nécessaire à cause des homonymes. */
	@Id
	private String id;
	
	private String nom;
	
	@Temporal(TemporalType.DATE)
	private Date dateNaissance;

	@XmlAttribute(name="id", required=true)
	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	@XmlElement(name="nom")
	public String getNom() {
		return nom;
	}

	public void setNom(String nom) {
		this.nom = nom;
	}

	@XmlTransient
	public Date getDateNaissance() {
		return dateNaissance;
	}

	public void setDateNaissance(Date dateNaissance) {
		this.dateNaissance = dateNaissance;
	}
	
	@SuppressWarnings("unused")
	@XmlElement(name="dateNaissance")
	@XmlSchemaType(name="date")
	private XMLGregorianCalendar getDN() {
		Date d = getDateNaissance();
		if (d==null) {
			return null;
		}
		GregorianCalendar gc = new GregorianCalendar();
		gc.setTime(d);
		try {
			return DatatypeFactory.newInstance().newXMLGregorianCalendar(gc);
		} catch (DatatypeConfigurationException e) {
			throw new Error(e);
		}
	}
	
	@SuppressWarnings("unused")
	private void setDN(XMLGregorianCalendar d) {
		setDateNaissance(d.toGregorianCalendar().getTime());
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Personne other = (Personne) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	
	
}
