package tiw5.modele;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlTransient;

@Entity
public class Role {

	// Pour JPA/JAXB
	@SuppressWarnings("unused")
	private Role() {
	}

	Role(Personne personne, Film film) {
		super();
		this.personne = personne;
		this.film = film;
		film.rawAddRole(this);
	}

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private long id;

	@ManyToOne
	private Personne personne;

	@ManyToOne
	private Film film;

	private boolean star;

	private String personnage;

	public long getId() {
		return id;
	}

	@XmlTransient
	public Personne getPersonne() {
		return personne;
	}

	public void setPersonne(Personne personne) {
		this.personne = personne;
	}

	@XmlTransient
	public Film getFilm() {
		return film;
	}

	public void setFilm(Film film) {
		this.film.rawRemoveRole(this);
		this.film = film;
		film.rawAddRole(this);
	}

	void rawSetFilm(Film film) {
		this.film = film;
	}

	@XmlAttribute(name = "star")
	public boolean isStar() {
		return star;
	}

	public void setStar(boolean star) {
		this.star = star;
	}

	@XmlElement(name = "personnage")
	public String getPersonnage() {
		return personnage;
	}

	public void setPersonnage(String personnage) {
		this.personnage = personnage;
	}

	@XmlAttribute(name = "personneId", required = true)
	@SuppressWarnings("unused")
	private String getPersonneId() {
		return getPersonne().getId();
	}

	@SuppressWarnings("unused")
	private void setPersonneId(String id) {
		personne = new Personne(id, null);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (int) (id ^ (id >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Role other = (Role) obj;
		if (id != other.id)
			return false;
		return true;
	}
}
