package tiw5.modele.dao;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;

import tiw5.modele.Film;
import tiw5.modele.Personne;
import tiw5.modele.Role;
import tiw5.modele.Societe;

public class FilmDAO {

	private EntityManager em;

	private PersonneDAO pDao;

	private SocieteDAO sDao;

	@PersistenceContext
	public void setEntityManager(EntityManager em) {
		this.em = em;
	}

	@Resource
	public void setPersonneDAO(PersonneDAO pDao) {
		this.pDao = pDao;
		pDao.setEntityManager(em);
	}

	@Resource
	public void setSocieteDAO(SocieteDAO sDao) {
		this.sDao = sDao;
		sDao.setEntityManager(em);
	}

	public FilmDAO() {
	}

	public FilmDAO(EntityManager em, PersonneDAO pDao, SocieteDAO sDao) {
		setEntityManager(em);
		setPersonneDAO(pDao);
		setSocieteDAO(sDao);
	}

	public Film getFilm(long id) {
		return em.find(Film.class, id);
	}

	public Film getFilm(String titre, int annee) {
		try {
			Film f = em.createNamedQuery("getFilmTitreAnnee", Film.class)
					.setParameter("titre", titre).setParameter("annee", annee)
					.getSingleResult();
			return f;
		} catch (NoResultException e) {
			return null;
		}
	}

	public Film updateFilm(Film updt) {
		Film f2 = getFilm(updt.getTitre(), updt.getAnnee());
		if (f2 == null) {
			updt.setId(0);
			List<Personne> realisateurs = new ArrayList<Personne>();
			realisateurs.addAll(updt.getRealisateurs());
			for (Personne p : realisateurs) {
				updt.retireRealisateur(p);
				updt.ajouteRealisateur(pDao.updatePersonne(p));
			}
			for (Role r : updt.getDistribution()) {
				r.setPersonne(pDao.updatePersonne(r.getPersonne()));
			}
			List<Societe> societes = new ArrayList<Societe>();
			societes.addAll(updt.getProduction());
			for (Societe s : societes) {
				updt.removeSocieteProduction(s);
				updt.ajouteSocieteProduction(sDao.updateSociete(s));
			}
			em.persist(updt);
			return updt;
		} else {
			for (Role r : updt.getDistribution()) {
				r.setPersonne(pDao.updatePersonne(r.getPersonne()));
				boolean found = false;
				for (Role r2 : f2.getDistribution()) {
					if (r2.getPersonne() == r.getPersonne()) {
						found = true;
						if (r.isStar()) {
							r2.setStar(true);
						}
						if (r2.getPersonnage() == null) {
							r2.setPersonnage(r.getPersonnage());
						}
					}
				}
				if (!found) {
					Role r2 = f2.ajouteRole(r.getPersonne());
					r2.setPersonnage(r.getPersonnage());
					r2.setStar(r.isStar());
				}
			}
			for (Personne p : updt.getRealisateurs()) {
				p = pDao.updatePersonne(p);
				if (!f2.getRealisateurs().contains(p)) {
					f2.ajouteRealisateur(p);
				}
			}
			for (Societe s : updt.getProduction()) {
				s = sDao.updateSociete(s);
				if(!f2.getProduction().contains(s)) {
					f2.ajouteSocieteProduction(s);
				}
			}
			return f2;
		}
	}
}
