package tiw5.modele;

import java.io.IOException;
import java.util.GregorianCalendar;

import javax.xml.XMLConstants;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.util.JAXBSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import junit.framework.TestCase;

import org.xml.sax.SAXException;

public class TestXMLMappings extends TestCase {

	private Validator validator;

	/*
	 * (non-Javadoc)
	 * 
	 * @see junit.framework.TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		if (validator == null) {
			validator = SchemaFactory
					.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI)
					.newSchema(
							new StreamSource(Film.class
									.getResourceAsStream("/films.xsd")))
					.newValidator();
		}
	}

	public void testFilmMapping() throws SAXException, IOException, JAXBException {
		Personne artiste = new Personne("urn:personne:p1", "Toto");
		Film film = new Film("Un bon debut", 2012);
		long filmId = 345;
		film.setId(filmId);
		film.ajouteRealisateur(artiste);
		film.ajouteRole(artiste);
		film.ajouteSocieteProduction(new Societe("Blabla prod."));
		validator.validate(new JAXBSource(JAXBContext.newInstance(Film.class), film));
	}
	
	public void testSocieteMapping() throws SAXException, IOException, JAXBException {
		Societe s = new Societe("Toto Inc.");
		s.setAdresse("Villeurbanne");
		s.setPays("France");
		validator.validate(new JAXBSource(JAXBContext.newInstance(Societe.class), s));
	}
	
	public void testPersonneMapping() throws SAXException, IOException, JAXBException {
		Personne p = new Personne("urn:personne:p1", "Toto");
		GregorianCalendar gc = new GregorianCalendar();
		gc.clear();
		gc.set(1980, 01, 01);
		p.setDateNaissance(gc.getTime());
		validator.validate(new JAXBSource(JAXBContext.newInstance(Personne.class), p));
	}
	
	public void testInfosFilms() throws SAXException, IOException, JAXBException {
		InfosFilms inf = new InfosFilms();
		Personne artiste = new Personne("urn:personne:p1", "Toto");
		Film film = new Film("Un bon debut", 2012);
		long filmId = 345;
		film.setId(filmId);
		film.ajouteRealisateur(artiste);
		film.ajouteRole(artiste);
		Societe s = new Societe("Blabla prod.");
		film.ajouteSocieteProduction(s);
		inf.addFilm(film);
		inf.addPersonne(artiste);
		inf.addSociete(s);
		validator.validate(new JAXBSource(JAXBContext.newInstance(InfosFilms.class), inf));
	}
}
