package tiw5.services;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebResult;
import javax.jws.WebService;

import org.springframework.transaction.annotation.Transactional;

import tiw5.modele.Film;
import tiw5.modele.InfosFilms;
import tiw5.modele.Personne;
import tiw5.modele.Role;
import tiw5.modele.Societe;
import tiw5.modele.dao.FilmDAO;
import tiw5.modele.dao.PersonneDAO;
import tiw5.modele.dao.SocieteDAO;

@WebService(name = "filmsInfosItf", portName = "filmsInfosPort", serviceName = "filmsInfosService", targetNamespace = "http://www.univ-lyon1.fr/M2TI/films/services")
public class FilmsInfosService {

	@Resource
	private FilmDAO fDao;

	@Resource
	PersonneDAO pDao;

	@Resource
	SocieteDAO sDao;

	@WebMethod(operationName = "getInfos", action = "getInfos")
	@Transactional
	public InfosFilms getInfos(@WebParam(name = "id") long filmId) {
		Film film = fDao.getFilm(filmId);
		InfosFilms inf = new InfosFilms();
		for (Personne p : film.getRealisateurs()) {
			inf.addPersonne(p);
		}
		for (Role r : film.getDistribution()) {
			if (!inf.getPersonnes().contains(r.getPersonne())) {
				inf.addPersonne(r.getPersonne());
			}
		}
		for (Societe s : film.getProduction()) {
			inf.addSociete(s);
		}
		inf.addFilm(film);
		return inf;
	}

	@WebMethod(operationName = "updateFilm", action = "updateFilm")
	@Transactional
	public @WebResult(name = "film")
	List<Film> updateFilm(@WebParam(name = "updateInfos") InfosFilms infosFilm) {
		List<Film> result = new ArrayList<Film>();
		for (Film updt : infosFilm.getFilms()) {
			Film f = fDao.updateFilm(updt);
			result.add(f);
		}
		for (Personne p : infosFilm.getPersonnes()) {
			pDao.updatePersonne(p);
		}
		for (Societe s : infosFilm.getSocietes()) {
			sDao.updateSociete(s);
		}
		return result;
	}
}
